@@ -13,8 +13,10 @@ import bz2
 import gzip
 import lzma
 
+from texta_face_analyzer.face_analyzer import FaceAnalyzer
+
 from texta_parsers.email_parser import EmailParser
-from texta_parsers.settings import META_FIELD
+from texta_parsers.settings import META_FIELD, FACE_VECTOR_FIELD_NAME
 from texta_parsers.tools.extension import Extension
 from texta_parsers.tools.scanner import DocScanner
 from texta_parsers.tools import rar_extractor
@@ -26,6 +28,9 @@ from . import exceptions
 # for digidoc parser
 FNULL = open(os.devnull, "w")
 
+# initiate face analyzer
+FACE_ANALYZER = FaceAnalyzer()
+
 
 class DocParser:
     
@@ -33,6 +38,7 @@ class DocParser:
         save_attachments=False,
         save_mails=False,
         save_documents=False,
+        parse_attachments=True,
         save_path = "parsed_files",
         temp_dir="", 
         languages=["est", "eng", "rus"],
@@ -43,6 +49,7 @@ class DocParser:
         :param: bool save_attachments: Whether to save email attachments.
         :param: bool save_mails: Whether to save emails.
         :param: bool save_documents: Whether to save files (other than emails and attachments).
+        :param: bool parse_attachments: Whether to parse the content of attached files in email.
         :param: str save_path: Base directory for files to be saved permanently.
         :param: str temp_dir: Base directory for files to be saved temporary.
         :param: [str] languages: Tika OCR languages.
@@ -54,6 +61,7 @@ class DocParser:
         self.save_attachments = save_attachments
         self.save_mails = save_mails
         self.save_documents = save_documents
+        self.parse_attachments = parse_attachments
         self.save_path = save_path
         self.max_file_size = max_file_size
         self.temp_dir_path = temp_dir
@@ -196,10 +204,15 @@ class DocParser:
     @staticmethod
     def _parse_file(document, languages):
         """
-        Parses document using TIKA and Speech2Text
+        Parses document using TIKA (for everything) and FaceAnalyzer (for images).
         """
-        output_documents = []
-        # TODO: WTF HAPPENS WITH OCR?
+        output_document = {}
+        # extract faces if an image
+        if document["extension"] in Extension.IMAGE_EXTENSIONS:
+            face_vectors = FACE_ANALYZER.vectorize_photo(document["path"])
+            if face_vectors:
+                output_document[FACE_VECTOR_FIELD_NAME] = [{"name": "UNKOWN_FACE", "vector": v.tolist()} for v in face_vectors]
+        # extract text using TIKA
         tika_output = parser.from_file(document["path"], requestOptions={"headers": {"X-Tika-OCRLanguage": "+".join(languages)}})
         content = tika_output["content"]
         if content != None:  # remove leading and trailing spacing
@@ -207,9 +220,9 @@ class DocParser:
             content = "\n".join(line for line in lines if line)
         else:
             content = ""
-
-        output_documents.append({"text": content})
-        return output_documents
+        output_document["text"] = content
+        # return as list
+        return [output_document]
 
     @staticmethod
     def _file_size_ok(max_file_size, document):
@@ -371,21 +384,40 @@ class DocParser:
                         # attachments in attachment_dicts are not parsed and the field content only contains raw payload in bytes.
                         # thus we need to call parse function on each content to get the text.
                         # moreover, each payload can potentially contain multiple files if it is an archive, for instance.
+
                         extracted_attachments = []
                         for attachment in attachment_dicts:
                             if("subject" in attachment): #is actually mail, do not parse it (already parsed!)
                                 attachment[META_FIELD] = meta.copy()
                                 extracted_attachments.append(attachment)
                             else:
-                                extracted_attachments += self._parse_attachment(attachment)
+                                if(self.parse_attachments == True):
+                                    try:
+                                        extracted_attachments += self._parse_attachment(attachment)
+                                    except:
+                                        exc_type, value, _ = sys.exc_info()
+                                        error_msg = "{}: {}".format(exc_type.__name__, value)
+                                        attachment[META_FIELD]["errors"] = "{}: {}".format(exc_type.__name__, value)
+                                        extracted_attachments += [attachment]
 
-                        # save attachments if specified
-                        for attachment in extracted_attachments:
-                            if("subject" not in attachment): #is actually mail, do not save it (already in email_parser if specified so!)
-                                self.save_attachment(attachment)
-                        
-                        yield msg_dict, extracted_attachments
 
+                                    # save attachments if specified
+                                    for attachment in extracted_attachments:
+                                        if("subject" not in attachment): #is actually mail, do not save it (already in email_parser if specified so!)
+                                            self.save_attachment(attachment)
+
+                                else:
+                                    for attachment in attachment_dicts:
+                                        attachment[META_FIELD] = meta.copy()
+                                        if("subject" not in attachment):
+                                            if("content" in attachment):
+                                                attachment.pop("content")
+                                            self.save_attachment(attachment)
+
+
+                        yield msg_dict, extracted_attachments if self.parse_attachments else attachment_dicts
+
+                # parse collections
                 elif (meta["extension"] in Extension.COLLECTION_EXTENSIONS):
                     if(save_to_file):
                         self.save_document(meta)
@@ -399,7 +431,8 @@ class DocParser:
                         error_msg = "{}: {}".format(exc_type.__name__, value)
                         meta["errors"] = error_msg
                         yield {META_FIELD: meta}
-                    
+
+                # parse everything else
                 else:
                     if(save_to_file):
                         self.save_document(meta)
